"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCommentParts = getCommentParts;
exports.isFile = isFile;
const fs = __importStar(require("fs"));
const markdown_1 = require("../../../libs/markdown");
function getCommentParts(model) {
    const md = [];
    for (const part of model) {
        switch (part.kind) {
            case 'text':
                md.push(part.text);
                break;
            case 'code':
                md.push(part.text);
                break;
            case 'inline-tag':
                switch (part.tag) {
                    case '@label':
                    case '@inheritdoc':
                        break;
                    case '@link':
                    case '@linkcode':
                    case '@linkplain': {
                        if (part.target) {
                            const url = getUrl(part);
                            const wrap = part.tag === '@linkcode' ? '`' : '';
                            md.push(url
                                ? `${(0, markdown_1.link)(`${wrap}${part.text}${wrap}`, this.getRelativeUrl(url))}`
                                : part.text);
                        }
                        else {
                            md.push(part.text);
                        }
                        break;
                    }
                    default:
                        md.push(`{${part.tag} ${part.text}}`);
                        break;
                }
                break;
            case 'relative-link':
                switch (typeof part.target) {
                    case 'number':
                        {
                            const reflection = this.page.project.files.resolve(part.target);
                            if (typeof reflection === 'object' && reflection.url) {
                                md.push(this.getRelativeUrl(reflection.url));
                                break;
                            }
                            const fileName = this.page.project.files.getName(part.target);
                            if (fileName) {
                                md.push(this.getRelativeUrl(`_media/${fileName}`));
                                break;
                            }
                        }
                        break;
                    case 'undefined':
                        md.push(part.text);
                        break;
                }
                break;
            default:
                md.push('');
        }
    }
    return md.join('');
}
function getUrl(part) {
    if (part.target.url) {
        return part.target.url;
    }
    if (part.target?.parent?.url) {
        return part.target?.parent?.url;
    }
    return typeof part.target === 'string' ? part.target : '';
}
function isFile(file) {
    try {
        return fs.statSync(file).isFile();
    }
    catch {
        return false;
    }
}
