"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.reflectionIndex = reflectionIndex;
const markdown_1 = require("../../../libs/markdown");
const typedoc_1 = require("typedoc");
function reflectionIndex(model, options) {
    const md = [];
    if (model.categories) {
        model.categories.forEach((categoryGroup) => {
            md.push((0, markdown_1.heading)(options.headingLevel, categoryGroup.title) + '\n');
            if (categoryGroup.description) {
                md.push(this.helpers.getCommentParts(categoryGroup.description) + '\n');
            }
            md.push(this.helpers.getGroupIndex(categoryGroup) + '\n');
        });
    }
    else {
        const groups = model.groups?.filter((group) => group.allChildrenHaveOwnDocument() &&
            group.title !== typedoc_1.ReflectionKind.pluralString(typedoc_1.ReflectionKind.Document));
        groups?.forEach((reflectionGroup) => {
            if (reflectionGroup.categories) {
                md.push((0, markdown_1.heading)(options.headingLevel, reflectionGroup.title) + '\n');
                reflectionGroup.categories.forEach((categoryGroup) => {
                    md.push((0, markdown_1.heading)(options.headingLevel + 1, categoryGroup.title) + '\n');
                    if (categoryGroup.description) {
                        md.push(this.helpers.getCommentParts(categoryGroup.description) + '\n');
                    }
                    md.push(this.helpers.getGroupIndex(categoryGroup) + '\n');
                });
            }
            else {
                md.push((0, markdown_1.heading)(options.headingLevel, reflectionGroup.title) + '\n');
                md.push(this.helpers.getGroupIndex(reflectionGroup) + '\n');
            }
        });
    }
    return md.join('\n');
}
